//==============================================================================
// rwm_maze.inc
// 02/04/2012
// Robert W. McGregor, www.mcgregorfineart.com
//------------------------------------------------------------------------------ 
// Description: Builds random, perfect rectangular mazes
//==============================================================================

#ifndef(RWM_MAZE_INC_TEMP)
#declare RWM_MAZE_INC_TEMP = version;

#include "shapes.inc"
#include "rand.inc"  

#ifndef (R_Maze) #declare R_Maze = seed(0); #end

//------------------------------------------------------------------------------ 
// Macro: CheckCorners()
//------------------------------------------------------------------------------ 

#macro CheckCorners(currR, currC)
   // upper left
   #if ((currR=0)&(currC=0)) 
      // e, s
      #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4))  
         CheckES(currR, currC)
      #else 
         #if (CountWallsUp(currR,currC+1)=4) 
            CheckE(currR, currC)
         #else
            #if (CountWallsUp(currR+1,currC)=4) 
               CheckS(currR, currC)
            #else
               Backtrack()
            #end
         #end
      #end
   #end

   // upper right
   #if ((currR=0)&(currC=lastCol)) 
      // s, w
      #if ((CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4))  
         CheckSW(currR, currC)
      #else 
         #if (CountWallsUp(currR+1,currC)=4) 
            CheckS(currR, currC)
         #else
            #if (CountWallsUp(currR,currC-1)=4) 
               CheckW(currR, currC)
            #else
               Backtrack()
            #end
         #end
      #end
   #end

   // lower left
   #if ((currR=lastRow)&(currC=0)) 
      // n, e
      #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4))  
         CheckNE(currR, currC)
      #else 
         #if (CountWallsUp(currR-1,currC)=4) 
            CheckN(currR, currC)
         #else
            #if (CountWallsUp(currR,currC+1)=4) 
               CheckE(currR, currC)
            #else
               Backtrack()
            #end
         #end
      #end
   #end

   // lower right
   #if ((currR=lastRow)&(currC=lastCol)) 
      // w, n
      #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4))  
         CheckWN(currR, currC)
      #else 
         #if (CountWallsUp(currR,currC-1)=4) 
            CheckW(currR, currC)
         #else
            #if (CountWallsUp(currR-1,currC)=4) 
              CheckN(currR, currC)
            #else
               Backtrack()
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckStartRow()
//------------------------------------------------------------------------------ 

#macro CheckStartRow(currR, currC)
   // start row & not corner, can move left, right or down
   #if (currR=0) 
      // e, s, w
      #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC+1)=4))
         CheckESW(currR, currC)
      #else 
         // e, w
         #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR,currC-1)=4))
            CheckEW(currR, currC)
         #else 
            // s, w
            #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR+1,currC)=4)) 
               CheckSW(currR, currC)
            #else 
               // e, s
               #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4)) 
                  CheckES(currR, currC)
               #else
                  // only one left unvisited - find it
                  #if (CountWallsUp(currR,currC-1)=4)
                     CheckW(currR, currC)
                  #else
                     #if (CountWallsUp(currR,currC+1)=4)
                        CheckE(currR, currC)
                     #else
                        #if (CountWallsUp(currR+1,currC)=4)
                           CheckS(currR, currC)
                        #else
                           Backtrack()  // dead end
                        #end 
                     #end 
                  #end
               #end
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckStartCol()
//------------------------------------------------------------------------------ 

#macro CheckStartCol(currR, currC)
   // start row & not corner, can move up, down or right
   #if (currC=0) 
      // n, e, s
      #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4))
         CheckNES(currR, currC)
      #else 
         // n, s
         #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR+1,currC)=4))
            CheckNS(currR, currC)
         #else 
            // e, s
            #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4))
               CheckES(currR, currC)
            #else 
               // n, e
               #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4)) 
                  CheckNE(currR, currC)
               #else
                  // only one left unvisited - find it
                  #if (CountWallsUp(currR-1,currC)=4)
                     CheckN(currR, currC)
                  #else
                     #if (CountWallsUp(currR+1,currC)=4)
                        CheckS(currR, currC)
                    #else
                        #if (CountWallsUp(currR+1,currC)=4)
                           CheckE(currR, currC)
                        #else
                           Backtrack()  // dead end
                        #end 
                     #end 
                  #end
               #end
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckLastCol()
//------------------------------------------------------------------------------ 

#macro CheckLastCol(currR, currC)
   // start row & not corner, can move up, down or left
   #if (currC=lastCol) 
      // s, w, n
      #if ((CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4)) 
         CheckSWN(currR, currC)
      #else 
         // n, s
         #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR+1,currC)=4)) 
            CheckNS(currR, currC)
         #else 
            // s, w
            #if ((CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4)) 
               CheckSW(currR, currC)
            #else 
               // w, n
               #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4)) 
                  CheckWN(currR, currC)
               #else
                  // only one left unvisited - find it
                  #if (CountWallsUp(currR-1,currC)=4)
                     CheckN(currR, currC)
                  #else
                     #if (CountWallsUp(currR+1,currC)=4)
                        CheckS(currR, currC)
                     #else
                        #if (CountWallsUp(currR+1,currC)=4)
                           CheckW(currR, currC)
                        #else
                           Backtrack()  // dead end
                        #end 
                     #end 
                  #end
               #end
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckLastRow()
//------------------------------------------------------------------------------ 

#macro CheckLastRow(currR, currC)
   // End row & not corner
   #if (currR=lastRow) 
      // w, n, e
      #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4))
         CheckWNE(currR,currC)
      #else 
         // e, w
         #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR,currC-1)=4)) 
            CheckEW(currR,currC)
         #else 
            // w, n
            #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4))
               CheckWN(currR,currC)
            #else 
               // n, e
               #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4)) 
                  CheckNE(currR,currC)
               #else
                  // only one left unvisited - find it
                  #if (CountWallsUp(currR,currC-1)=4)
                     CheckW(currR,currC)
                  #else      
                     #if (CountWallsUp(currR,currC+1)=4)
                        CheckE(currR,currC)
                     #else
                        #if (CountWallsUp(currR-1,currC)=4)
                           CheckN(currR,currC)
                        #else
                           Backtrack()  // dead end
                        #end 
                     #end 
                  #end
               #end
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckInterior()
//------------------------------------------------------------------------------ 

#macro CheckInterior(currR, currC)
   // n, e, s, w 
   #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4)) 
      CheckNESW(currR, currC)
   #else
      // n, e, s
      #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4))
         CheckNES(currR, currC)
      #else
         // e, s, w
         #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4))
            CheckESW(currR, currC)
         #else
            // s, w, n
            #if ((CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4)) 
               CheckSWN(currR, currC)
            #else
               // w, n, e
               #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4))
                  CheckWNE(currR,currC)
               #else 
                  // n, s
                  #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR+1,currC)=4))
                     CheckNS(currR, currC)
                  #else 
                     // e, w
                     #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR,currC-1)=4))
                        CheckEW(currR, currC)
                     #else 
                        // n, e
                        #if ((CountWallsUp(currR-1,currC)=4)&(CountWallsUp(currR,currC+1)=4))  
                           CheckNE(currR, currC)
                        #else 
                           // e, s
                           #if ((CountWallsUp(currR,currC+1)=4)&(CountWallsUp(currR+1,currC)=4))  
                              CheckES(currR, currC)
                           #else 
                              // s, w
                              #if ((CountWallsUp(currR+1,currC)=4)&(CountWallsUp(currR,currC-1)=4))  
                                 CheckSW(currR, currC)
                              #else 
                                 // w, n
                                 #if ((CountWallsUp(currR,currC-1)=4)&(CountWallsUp(currR-1,currC)=4))  
                                    CheckWN(currR, currC)
                                 #else 
                                    // only one left unvisited - find it
                                    #if (CountWallsUp(currR-1,currC)=4)
                                       CheckN(currR, currC)
                                    #else
                                       #if (CountWallsUp(currR,currC+1)=4)
                                          CheckE(currR, currC)
                                       #else
                                          #if (CountWallsUp(currR+1,currC)=4)
                                             CheckS(currR, currC)
                                          #else
                                             #if (CountWallsUp(currR,currC-1)=4)
                                                CheckW(currR, currC)
                                             #else
                                                Backtrack()  // dead end
                                             #end               
                                          #end               
                                       #end               
                                    #end               
                                 #end               
                              #end               
                           #end               
                        #end               
                     #end
                  #end
               #end
            #end
         #end
      #end
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckBorders()
//------------------------------------------------------------------------------ 

#macro CheckBorders(currR, currC)
   #if (!bFoundRandomNeighbor) CheckStartRow(currR, currC) #end
   #if (!bFoundRandomNeighbor) CheckLastRow(currR, currC)   #end
   #if (!bFoundRandomNeighbor) CheckStartCol(currR, currC) #end
   #if (!bFoundRandomNeighbor) CheckLastCol(currR, currC)   #end
   
#end

//------------------------------------------------------------------------------ 
// Macro: RemoveRandomWall()
//------------------------------------------------------------------------------ 
// Find a random, unvisited neighbor cell:
//   * corners have 2 neighbors
//   * start and end ros/cols have 3 neighbors
//   * all others have 4 neighbors
//------------------------------------------------------------------------------ 

#macro RemoveRandomWall(currR, currC)
   // corners first
   #if (!bFoundRandomNeighbor) CheckCorners(currR, currC)  #end
   #if (!bFoundRandomNeighbor) CheckBorders(currR, currC)  #end
   #if (!bFoundRandomNeighbor) CheckInterior(currR, currC) #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckNESW()
//------------------------------------------------------------------------------ 
#macro CheckNESW(currR, currC)
   #local n = int(RRand(1,4,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallNorth(currR-1,currC)   // remove north wall
         #break
      #case (2)
         RemoveWallEast(currR,currC+1)    // remove east wall
         #break
      #case (3)
         RemoveWallSouth(currR+1,currC)   // remove south wall
         #break
      #case (4)
         RemoveWallWest(currR,currC-1)    // remove west wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckNES()
//------------------------------------------------------------------------------ 
#macro CheckNES(currR, currC)
   #local n = int(RRand(1,3,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallNorth(currR-1,currC)   // remove north wall
         #break
      #case (2)
         RemoveWallEast(currR,currC+1)    // remove east wall
         #break
      #case (3)
         RemoveWallSouth(currR+1,currC)   // remove south wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckESW()
//------------------------------------------------------------------------------ 
#macro CheckESW(currR, currC)
   #local n = int(RRand(1,3,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallEast(currR,currC+1)   // remove east wall
         #break
      #case (2)
         RemoveWallSouth(currR+1,currC)  // remove south wall
         #break
      #case (3)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckSWN()
//------------------------------------------------------------------------------ 
#macro CheckSWN(currR, currC)
   #local n = int(RRand(1,3,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallSouth(currR+1,currC)  // remove south wall
         #break
      #case (2)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
      #case (3)
         RemoveWallNorth(currR-1,currC)  // remove north wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckWNE()
//------------------------------------------------------------------------------ 
#macro CheckWNE(currR, currC)
   #local n = int(RRand(1,3,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
      #case (2)
         RemoveWallNorth(currR-1,currC)  // remove north wall
         #break
      #case (3)
         RemoveWallEast(currR,currC+1)   // remove east wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckNS()
//------------------------------------------------------------------------------ 
#macro CheckNS(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallNorth(currR-1,currC)   // remove north wall
         #break
      #case (2)
         RemoveWallSouth(currR+1,currC)   // remove south wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckEW()
//------------------------------------------------------------------------------ 
#macro CheckEW(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallEast(currR,currC+1)   // remove east wall
         #break
      #case (2)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckNE()
//------------------------------------------------------------------------------ 
#macro CheckNE(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallNorth(currR-1,currC)  // remove north wall
         #break
      #case (2)
         RemoveWallEast(currR,currC+1)  // remove east wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckES()
//------------------------------------------------------------------------------ 
#macro CheckES(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallEast(currR,currC+1)    // remove east wall
         #break
      #case (2)
         RemoveWallSouth(currR+1,currC)   // remove south wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckSW()
//------------------------------------------------------------------------------ 
#macro CheckSW(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallSouth(currR+1,currC)  // remove south wall
         #break
      #case (2)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckWN()
//------------------------------------------------------------------------------ 
#macro CheckWN(currR, currC)
   #local n = int(RRand(1,2,R_Maze)+0.5);
   #switch (n)
      #case (1)
         RemoveWallWest(currR,currC-1)   // remove west wall
         #break
      #case (2)
         RemoveWallNorth(currR-1,currC)  // remove north wall
         #break
   #end
#end

//------------------------------------------------------------------------------ 
// Macro: CheckN()
//------------------------------------------------------------------------------ 
#macro CheckN(currR, currC)
   RemoveWallNorth(currR-1,currC) // remove north wall
#end

//------------------------------------------------------------------------------ 
// Macro: CheckE()
//------------------------------------------------------------------------------ 
#macro CheckE(currR, currC)
   RemoveWallEast(currR,currC+1)   // remove east wall
#end

//------------------------------------------------------------------------------ 
// Macro: CheckS()
//------------------------------------------------------------------------------ 
#macro CheckS(currR, currC)
   RemoveWallSouth(currR+1,currC)   // remove south wall
#end

//------------------------------------------------------------------------------ 
// Macro: CheckW()
//------------------------------------------------------------------------------ 
#macro CheckW(currR, currC)
   RemoveWallWest(currR,currC-1)   // remove west wall
#end

//------------------------------------------------------------------------------ 
// Macro: RemoveWallNorth()
//------------------------------------------------------------------------------
#macro RemoveWallNorth(r, c)
   #declare currRow = r;          // set new current location
   #local prevRow = r+1;
   #local prevCol = c;
   
   #local vec = aWalls[prevRow][prevCol];
   #local vec = <0, vec.y, vec.z, vec.t>; // north wall is down in prev cell
   #declare aWalls[prevRow][prevCol] = vec;
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, vec.y, 0, vec.t>; // south wall is down in curr cell
   #declare aWalls[r][c] = vec;
   
   // only do prev the first time through, to enable backtracking
   #if ((aPrevCell[r][c].u = -1)&(aPrevCell[r][c].v = -1)) 
      #declare aPrevCell[r][c] = <prevRow, prevCol>; 
   #end  

   #declare bFoundRandomNeighbor = 1;
#end

//------------------------------------------------------------------------------ 
// Macro: RemoveWallEast()
//------------------------------------------------------------------------------
#macro RemoveWallEast(r, c)
   #declare currCol = c;          // set new current location
   #local prevRow = r;
   #local prevCol = c-1;
   
   #local vec = aWalls[prevRow][prevCol];
   #local vec = <vec.x, 0, vec.z, vec.t>;  // east wall is down in prev cell
   #declare aWalls[prevRow][prevCol] = vec;
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, vec.y, vec.z, 0>;  // west wall is down in curr cell
   #declare aWalls[r][c] = vec;                    
   
   // only do prev the first time through, to enable backtracking
   #if ((aPrevCell[r][c].u = -1)&(aPrevCell[r][c].v = -1)) 
      #declare aPrevCell[r][c] = <prevRow, prevCol>; 
   #end  
   
   #declare bFoundRandomNeighbor = 1;
#end           

//------------------------------------------------------------------------------ 
// Macro: RemoveWallSouth()
//------------------------------------------------------------------------------
#macro RemoveWallSouth(r, c)
   #declare currRow = r;          // set new current location
   #local prevRow = r-1;
   #local prevCol = c;
   
   #local vec = aWalls[prevRow][prevCol];
   #local vec = <vec.x, vec.y, 0, vec.t>;  // south wall is down in prev cell
   #declare aWalls[prevRow][prevCol] = vec;
   
   #local vec = aWalls[r][c];
   #local vec = <0, vec.y, vec.z, vec.t>;  // north wall is down in curr cell
   #declare aWalls[r][c] = vec;
   
   // only do prev the first time through, to enable backtracking
   #if ((aPrevCell[r][c].u = -1)&(aPrevCell[r][c].v = -1)) 
      #declare aPrevCell[r][c] = <prevRow, prevCol>; 
   #end  
   
   #declare bFoundRandomNeighbor = 1;
#end

//------------------------------------------------------------------------------ 
// Macro: RemoveWallWest()
//------------------------------------------------------------------------------
#macro RemoveWallWest(r, c)
   #declare currCol = c;          // set new current location
   #local prevRow = r;
   #local prevCol = c+1;
   
   #local vec = aWalls[prevRow][prevCol];
   #local vec = <vec.x, vec.y, vec.z, 0>;  // west wall is down in prev cell
   #declare aWalls[prevRow][prevCol] = vec;
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, 0, vec.z, vec.t>;  // east wall is down in curr cell
   #declare aWalls[r][c] = vec;
   
   // only do prev the first time through, to enable backtracking
   #if ((aPrevCell[r][c].u = -1)&(aPrevCell[r][c].v = -1)) 
      #declare aPrevCell[r][c] = <prevRow, prevCol>; 
   #end  
   
   #declare bFoundRandomNeighbor = 1;
#end

//------------------------------------------------------------------------------ 
// Macro: Backtrack()
//------------------------------------------------------------------------------
#macro Backtrack()
   // move back to previous cell
   #local cr = aPrevCell[currRow][currCol].u;
   #local cc = aPrevCell[currRow][currCol].v;

   #declare currRow = cr;
   #declare currCol = cc;      
   #declare bFoundRandomNeighbor = 1;
   #declare visitedCells = visitedCells - 1;
#end

//------------------------------------------------------------------------------ 
// Macro: RemoveBorderNorth()
//------------------------------------------------------------------------------
#macro RemoveBorderNorth(r, c)
   
   #local vec = aWalls[r][c];
   #local vec = <0, vec.y, vec.z, vec.t>; 
   #declare aWalls[r][c] = vec;                    
   
#end           

//------------------------------------------------------------------------------ 
// Macro: RemoveBorderEast()
//------------------------------------------------------------------------------
#macro RemoveBorderEast(r, c)
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, 0, vec.z, vec.t>;
   #declare aWalls[r][c] = vec;                    
   
#end           

//------------------------------------------------------------------------------ 
// Macro: RemoveBorderSouth()
//------------------------------------------------------------------------------
#macro RemoveBorderSouth(r, c)
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, vec.y, 0, vec.t>;
   #declare aWalls[r][c] = vec;                    
   
#end           

//------------------------------------------------------------------------------ 
// Macro: RemoveBorderWest()
//------------------------------------------------------------------------------
#macro RemoveBorderWest(r, c)
   
   #local vec = aWalls[r][c];
   #local vec = <vec.x, vec.y, vec.z, 0>;
   #declare aWalls[r][c] = vec;                    
   
#end           

//------------------------------------------------------------------------------ 
// Macro: CountWallsUp()
//------------------------------------------------------------------------------
#macro CountWallsUp(r, c)
   #local n = 0;
   #if (aWalls[r][c].x=1) #local n = n + 1; #end
   #if (aWalls[r][c].y=1) #local n = n + 1; #end
   #if (aWalls[r][c].z=1) #local n = n + 1; #end
   #if (aWalls[r][c].t=1) #local n = n + 1; #end
   (n)
#end

//------------------------------------------------------------------------------ 
// Macro: GenerateMaze()
//------------------------------------------------------------------------------
#macro GenerateMaze(rows, cols)
  
   #declare lastRow = rows-1;
   #declare lastCol = cols-1;
   
   // array for the maze cells
   #declare aWalls    = array[rows][cols];
   #declare aPrevCell = array[rows][cols]; // for backtracking
   
   // pick random entrance and exit
   #declare entranceRow = int(RRand(0,lastRow,R_Maze)+0.5);
   #if ((entranceRow=0)|(entranceRow=lastRow))
      // top or bottom row, any col will work
      #declare entranceCol = int(RRand(0,lastCol,R_Maze)+0.5);
   #else                
      // must use col 0 or lastCol (far left or far right)
      #local n = int(RRand(0,1,R_Maze)+0.5);
      #if (n=0)
         #declare entranceCol = 0;
      #else
         #declare entranceCol = lastCol;
      #end
   #end
   
   // make sure they're not the same
   #declare bEntranceAndExitFound = 0;
   
   #while (!bEntranceAndExitFound)
      #declare exitRow = int(RRand(0,lastRow,R_Maze)+0.5);
      #if ((exitRow=0)|(exitRow=lastRow))
         // top or bottom row, any col will work
         #declare exitCol = int(RRand(0,lastCol,R_Maze)+0.5);
      #else                
         // must use col 0 or lastCol (far left or far right)
         #local n = int(RRand(0,1,R_Maze)+0.5);
         #if (n=0)
            #declare exitCol = 0;
         #else
            #declare exitCol = lastCol;
         #end
      #end
      #if ((entranceRow!=exitRow)|(entranceCol!=exitCol)) #declare bEntranceAndExitFound = 1; #end
   #end
   
   // Each cell is a 4D vector <x,y,z,t> representing <N,E,S,W>
   // init each cell with all walls up
   #local r = 0;
   #while (r<rows)
      #local c = 0;
      #while (c<cols)
         #declare aWalls[r][c] = <1, 1, 1, 1>;
         #declare aPrevCell[r][c] = <-1,-1>;
         #local c = c + 1;
      #end
      #local r = r + 1;
   #end
   
   #declare currRow = entranceRow;
   #declare currCol = entranceCol;
   #declare prevRow = -1;
   #declare prevCol = -1;
   
   // knock down walls until maze is complete       
   #debug concat("\nKnock down walls until maze is complete...\n")
   #declare totalCells   = rows*cols;
   #declare visitedCells = 1;
   #declare bFoundRandomNeighbor = 0;
   
   #while (visitedCells < totalCells)
       RemoveRandomWall(currRow, currCol)
       #declare bFoundRandomNeighbor = 0;
       #declare visitedCells = visitedCells + 1;
       #if (mod(visitedCells,100)=0) #debug concat("visitedCells: ", str(visitedCells, 0, 0), "\n") #end
   #end
   
   // Knock down a wall to the outside for entrance
   #declare bEntranceOpen = 0;     
   #debug concat("entranceCell = <",str(entranceRow,0,0),", ", str(entranceCol,0,0),">\n")
   #if (entranceRow=0)     
      RemoveBorderNorth(entranceRow, entranceCol)
      #declare bEntranceOpen = 1;
   #end
   
   #if ((!bEntranceOpen)&(entranceCol=lastCol))
      RemoveBorderEast(entranceRow, entranceCol)
      #declare bEntranceOpen = 1;
   #end
   
   #if ((!bEntranceOpen)&(entranceRow=lastRow)) 
      RemoveBorderSouth(entranceRow, entranceCol)
      #declare bEntranceOpen = 1;
   #end
   
   #if ((!bEntranceOpen)&(entranceCol=0))      
      RemoveBorderWest(entranceRow, entranceCol)
   #end   
   
   // Knock down a wall to the outside for exit
   #declare bExitOpen = 0;
   #debug concat("exitCell = <",str(exitRow,0,0),", ", str(exitCol,0,0),">\n")
   #if (exitRow=0)     
      RemoveBorderNorth(exitRow, exitCol)
      #declare bExitOpen = 1;
   #end
   
   #if ((!bExitOpen)&(exitCol=lastCol))
      RemoveBorderEast(exitRow, exitCol)
      #declare bExitOpen = 1;
   #end
   
   #if ((!bExitOpen)&(exitRow=lastRow)) 
      RemoveBorderSouth(exitRow, exitCol)
      #declare bExitOpen = 1;
   #end
   
   #if ((!bExitOpen)&(exitCol=0))      
      RemoveBorderWest(exitRow, exitCol)
   #end   
   
#end

//------------------------------------------------------------------------------ 
// Macro: BuildMaze()
//------------------------------------------------------------------------------
#macro BuildMaze(rows, cols, wall_thickness)
   
   GenerateMaze(rows, cols)
   
   #declare Box_N = box { -0.5, 0.5 scale z*wall_thickness translate  <-0.5, 0,  0.5> }
   #declare Box_E = box { -0.5, 0.5 scale x*wall_thickness translate  < 0.5, 0, -0.5> }
   #declare Box_S = box { -0.5, 0.5 scale z*wall_thickness translate  < 0.5, 0, -0.5> }
   #declare Box_W = box { -0.5, 0.5 scale x*wall_thickness translate  <-0.5, 0,  0.5> }
   #declare Box_C = box { -0.5, 0.5 scale <wall_thickness, 1, wall_thickness> }
   
   Center_Object (
      union {
         #local r=0;
         #while (r<rows)
            #local c=0;
            #while (c<cols)
               #if (aWalls[r][c].x=1) object { Box_N translate <c+0.5, 0, -r>}     #end  // N
               #if (aWalls[r][c].y=1) object { Box_E translate <c,     0, -r+0.5>} #end  // E
               #if (aWalls[r][c].z=1) object { Box_S translate <c-0.5, 0, -r>}     #end  // S
               #if (aWalls[r][c].t=1) object { Box_W translate <c,     0, -r-0.5>} #end  // W
               
               object { Box_C translate <c+0.5, 0, -r+0.5> }
               object { Box_C translate <c-0.5, 0, -r-0.5> }
               object { Box_C translate <c+0.5, 0, -r-0.5> }
               object { Box_C translate <c-0.5, 0, -r+0.5> }
               
               #local c = c + 1;
            #end
            #local r = r + 1;
         #end
      },x+y+z
   )  
#end

#end // #ifndef(RWM_MAZE_INC_TEMP)
